/*
 * Fichier : menu.c
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour
 */

#include "common.h"
#include "menu.h"
#include "hal_LCD.h"
#include "Affepaper.h"


bool    AlarmeLed = false;                  // vrai si on fait cligonter la LED si alarme */
bool    AlarmeBuz = false;                  // vrai si on active le buzzer si alarme */
uint16_t SeuilAlarme = alarme_defaut;       // Seuil  partir duquel on gnre une alarme
const char T_ON[]  = "ON";
const char T_OFF[] = "OFF";
const char T_LED[] = "LED";
const char T_BUZ[] = "BUZ";
const char T_AL[]  = "AL ";

/*
 Gre le menu du compteur. Procdure non bloquante.
 Auteur : Pascal Chour

*/

void AfficheMenu()
{
    char T[10];
    switch (ETAT_MENU)
    {
        case MENU_PAPER:
                LCD_writeTxtLeft("EPAPER",6);
        break;
        case MENU_LED:
        case MENU_LED_change:
            strcpy(T,T_LED);
            if (AlarmeLed) {strcat(T," ");LCD_writeTxtLeft(strcat(T,T_ON),6);} else {LCD_writeTxtLeft(strcat(T,T_OFF),6);}
        break;
        case MENU_BUZ:
        case MENU_BUZ_change:
            strcpy(T,T_BUZ);
            if (AlarmeBuz) {strcat(T," ");LCD_writeTxtLeft(strcat(T,T_ON),6);} else {LCD_writeTxtLeft(strcat(T,T_OFF),6);}
        break;
        case MENU_ALARME:
        case MENU_ALARME_change:
            strcpy(T,T_AL);
            if (SeuilAlarme == 0) {
                LCD_writeTxtLeft(strcat(T,T_OFF),6);
            } else {
                LCD_writeTxtLeft(strcat(T,"   "),6);
                LCD_writeIntRight (SeuilAlarme);
            }
        break;
        case MENU_EXIT:
            LCD_writeTxtLeft("-EXIT-",6);
        break;
    }
}



void MENU_automate(unsigned int buttonPressed)
{
    while (Poussoir | Sens)
    {
      switch (ETAT_MENU)
      {
        case MENU_REPOS:
          if (Poussoir) {
              ETAT_MENU = MENU_PAPER; // Update e-paper
              AfficheMenu();
          }
        break;

        case MENU_PAPER:
            if (Poussoir) { // Si poussoir, on force la sortie et on force le rafraichissement du e-paper
                Sens = 0;
                ETAT_MENU = MENU_REPOS;
                CompteInactivite = 0;
                LCD_ClearText();
                ValideAlarme = true;
            } else {
                if (Sens < 0) {
                    Sens = Sens +1;
                    ETAT_MENU = MENU_EXIT;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens - 1;
                        ETAT_MENU = MENU_LED;
                        AfficheMenu();
                    }
                }
            }
         break;

        case MENU_LED:
            if (Poussoir) {
                ETAT_MENU = MENU_LED_change;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    ETAT_MENU = MENU_PAPER;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        ETAT_MENU = MENU_BUZ;
                        AfficheMenu();
                    }
                }
            }
        break;

        case MENU_LED_change:
            if (Poussoir) {
                ETAT_MENU = MENU_LED;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    AlarmeLed = !AlarmeLed;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        AlarmeLed = !AlarmeLed;
                        AfficheMenu();
                    }
                }
            }
        break;

        case MENU_BUZ:
            if (Poussoir) {
                ETAT_MENU = MENU_BUZ_change;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    ETAT_MENU = MENU_LED;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        ETAT_MENU = MENU_ALARME;
                        AfficheMenu();
                    }
                }
            }
        break;

        case MENU_BUZ_change:
            if (Poussoir) {
                ETAT_MENU = MENU_BUZ;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    AlarmeBuz = !AlarmeBuz;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        AlarmeBuz = !AlarmeBuz;
                        AfficheMenu();
                    }
                }
            }
        break;

        case MENU_ALARME:
            if (Poussoir) {
                ETAT_MENU = MENU_ALARME_change;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    ETAT_MENU = MENU_BUZ;
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        ETAT_MENU = MENU_EXIT;
                        AfficheMenu();
                    }
                }
            }
        break;

        case MENU_ALARME_change:
            if (Poussoir) {
                ETAT_MENU = MENU_ALARME;
            } else {
                if (Sens < 0) {
                    Sens = Sens+1;
                    if (SeuilAlarme > 1) {SeuilAlarme = SeuilAlarme-1;}
                    AfficheMenu();
                } else {
                    if (Sens > 0) {
                        Sens = Sens -1;
                        if(SeuilAlarme < 999) {SeuilAlarme = SeuilAlarme+1;}
                        AfficheMenu();
                    }
                }
            }
        break;

          case MENU_EXIT:
              if (Poussoir) {
                  ETAT_MENU = MENU_REPOS;
                  CompteInactivite = 0;
                  LCD_ClearText();
                  Sens = 0;
              } else {
                  if (Sens < 0) {
                      Sens = Sens+1;
                      ETAT_MENU = MENU_ALARME;
                      AfficheMenu();
                  } else {
                      if (Sens > 0) {
                          Sens = Sens -1;
                          ETAT_MENU = MENU_PAPER;
                          AfficheMenu();
                      }
                  }
              }
          break;
      }
      Poussoir = false;
   }
}



